/*
 * Copyright 2009 OW2 Chameleon
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.ow2.chameleon.json.test;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.util.List;
import java.util.Map;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.ow2.chameleon.json.impl.JSONServiceProvider;


public class FromFileTest extends TestCase {

    public static final String ROOT_STR = "src/test/resources";
    public static final File ROOT = new File(ROOT_STR);

    JSONServiceProvider svc;

    public void setUp() {
        svc = new JSONServiceProvider();
    }

    private InputStream getInputStream(String name) throws FileNotFoundException {
        File file = new File(ROOT, name);
        return new FileInputStream(file);
    }

    public void testWithNull() throws ParseException, IOException {
        Map map = svc.fromJSON((InputStream) null);
        Assert.assertEquals(0, map.size());
    }

    public void testFromJSON() throws ParseException, FileNotFoundException, IOException {
        Map m1 = svc.fromJSON(getInputStream("SIMPLE.json"));
        Assert.assertEquals("foo", m1.get("foo"));
        Assert.assertEquals(1, m1.size());

        Map m2 = svc.fromJSON(getInputStream("SIMPLE_2.json"));
        Assert.assertEquals("foo", m2.get("foo"));
        Assert.assertEquals("bar", m2.get("bar"));
        Assert.assertEquals(2, m2.size());

        Map m3 = svc.fromJSON(getInputStream("EMPTY_JSON.json"));;
        Assert.assertEquals(0, m3.size());

        Map m4 = svc.fromJSON(getInputStream("EMPTY.json"));
        Assert.assertEquals(0, m4.size());

        Map m6 = svc.fromJSON(getInputStream("SIMPLE_3.json"));
        Assert.assertEquals(new Integer(1), m6.get("one"));
        Assert.assertEquals(new Double(2.4), m6.get("double"));
        Assert.assertEquals(Boolean.TRUE, m6.get("true"));
        Assert.assertEquals(Boolean.FALSE, m6.get("false"));
        Assert.assertEquals("foo", m6.get("foo"));
        Assert.assertEquals(svc.getNull(), m6.get("nil"));
        Assert.assertFalse(null == m6.get("nil")); // It's not null it's NULL (Javascript) !
        Map m62 = (Map) m6.get("object");
        Assert.assertEquals("foo", m62.get("foo"));
        Assert.assertEquals("bar", m62.get("bar"));
        Assert.assertEquals(2, m62.size());
    }

    public void testFromJSONWithArray() throws ParseException, FileNotFoundException, IOException {
        Map m1 = svc.fromJSON(getInputStream("COLLECTION.json"));
        Assert.assertEquals(1, m1.size());
        List list = (List) m1.get("items");
        Assert.assertEquals(3, list.size());
        Map m11 = (Map) list.get(0);
        Assert.assertEquals(new Integer(1), m11.get("x"));
        Assert.assertEquals(new Integer(1), m11.get("y"));

        // Let's analyze the third entry : {x=[{x=3}, []], y=3}
        Map e3 = (Map) list.get(2);
        List x = (List) e3.get("x"); // [{x=3}, []]
        Assert.assertEquals(2, x.size());
        Assert.assertEquals(new Integer(3), ((Map)x.get(0)).get("x"));
        Assert.assertEquals(0, ((List) x.get(1)).size());
        Assert.assertEquals(new Integer(3),  e3.get("y"));

    }

    public void testFromJSONOnStore() throws ParseException, FileNotFoundException, IOException {
        Map m1 = svc.fromJSON(getInputStream("STORE.json"));
        Assert.assertEquals(3, m1.size());
        List list = (List) m1.get("items");
        Assert.assertEquals(3, list.size());
        Map m11 = (Map) list.get(0);
        Assert.assertEquals("dd", m11.get("funktion"));
        Assert.assertEquals("dd", m11.get("nennweite"));
    }



}
